// 11-Motor.c
// gcc -Wall -o "%e" "%f" -llgpio -lpthread -g -O0 libMyPi2.a
// Geanyのオプションの"%e"は実行ファイル名, "%f"はソースファイル名のこと

#include <stdio.h>
#include <stdlib.h>         //EXIT_SUCCESS
#include <lgpio.h>          //lgGpiochipOpen,lgGpioWrite,etc
#include  "MyPi2.h"         //マイライブラリ
#define PI5         4       // /dev/gpiochip4
#define PI4B        0       // /dev/gpiochip0
#define SW_WHITE    27      //白色SW
#define MOT_RIGHT2  22      //右モータ BIN2
#define MOT_RIGHT1  23      //右モータ BIN1
#define MOT_LEFT2   24      //左モータ AIN2
#define MOT_LEFT1   25      //左モータ AIN1
#define MNUM        4       //モータの信号数
#define RUN_TIME    2       //動作時間２秒間
#define I2C_BUS     1       // /dev/i2c-1
#define LCD_ADR     0x3e    //LCD スレーブアドレス
#define LCD_IR      0x00    //インストラクションレジスタ

/* グローバル宣言  */
// モータ駆動IC信号線
const int outGpio[MNUM] = {MOT_RIGHT1,MOT_RIGHT2,MOT_LEFT1,MOT_LEFT2};

/* プロトタイプ宣言 */
void RightForward(int hnd);    //右モータ正転
void RightReverse(int hnd);    //右モータ逆転
void LeftForward(int hnd);     //左モータ正転
void LeftReverse(int hnd);     //左モータ逆転
void Forward(int hnd);         //左右のモータの正転
void Reverse(int hnd);         //左右のモータの逆転
void Stop(int hnd);            //左右のモータのストップ
void LcdClrMsg(int hndLcd, char *s); //LCDをクリアして表示

int main(void){
    int hnd,hndLcd;
    int lFlgOut=0;
    int lFlgIn=LG_SET_PULL_NONE;   //内部抵抗を使用しない
    int levels[MNUM]={0,0,0,0};    //出力にグループ化したGPIOの初期値
    hnd = lgGpiochipOpen(PI5);
    //hnd = lgGpiochipOpen(PI4B);
    lgGpioClaimInput(hnd,lFlgIn,SW_WHITE);    //白色SWを入力に設定
    lgGroupClaimOutput(hnd,lFlgOut,MNUM,outGpio,levels);
    hndLcd = lgI2cOpen(I2C_BUS,LCD_ADR,0);  //LCDのハンドルを取得
    LcdSetup(hndLcd);                       //LCDの初期化
    
    LcdClrMsg(hndLcd,"Push white SW.");
    while(1){
        if(lgGpioRead(hnd, SW_WHITE)==LG_HIGH){ //白色SWが押された

        RightForward(hnd);
        LcdClrMsg(hndLcd,"Right wheel moves forward.");
        lguSleep(RUN_TIME);

        RightReverse(hnd);
        LcdClrMsg(hndLcd,"Right wheel moves reverse.");
        lguSleep(RUN_TIME);

        LeftForward(hnd);
        LcdClrMsg(hndLcd,"Left wheel moves forward.");
        lguSleep(RUN_TIME);

        LeftReverse(hnd);
        LcdClrMsg(hndLcd,"Left wheel moves reverse.");
        lguSleep(RUN_TIME);

        Forward(hnd);
        LcdClrMsg(hndLcd,"All wheels move  forward.");
        lguSleep(RUN_TIME);

        Reverse(hnd);
        LcdClrMsg(hndLcd,"All wheels move  reverse.");
        lguSleep(RUN_TIME);

        Stop(hnd);
        LcdClrMsg(hndLcd,"All wheels stop.");
        lguSleep(RUN_TIME);

        LcdClrMsg(hndLcd,"Push white SW.");
        }
        lguSleep(0.001);    //CPU使用率の抑制のため 1ms
    }
    lgI2cClose(hndLcd);
    lgGpiochipClose(hnd);
    return EXIT_SUCCESS;
}

/* モータ関係の関数
  右モータ正転  void RightForward(int hnd)
  右モータ逆転  RightReverse(int hnd)
  左モータ正転  LeftForward(int hnd)
  左モータ逆転  void LeftReverse(int hnd)
  左右のモータの正転  void Forward(int hnd)
  左右のモータの逆転  void Reverse(int hnd)
  左右のモータのストップ  void Stop(int hnd)
  引  数   hnd  ハンドル
  戻り値   なし */
void RightForward(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void RightReverse(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void LeftForward(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_HIGH);
}
void LeftReverse(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void Forward(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_HIGH);
}
void Reverse(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void Stop(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT1,LG_HIGH);
}

/*関数名   void LcdClrMsg(int hndLcd, char *s)
  引  数   hndLcd  LCDのハンドル
  引  数   s  文字列へのポインタ
  戻り値   なし
  概  要   LCDをクリアして文字列を表示 */
void LcdClrMsg(int hndLcd, char *s){
    LcdClear(hndLcd);
    LcdWriteString(hndLcd, s);
}
