// 11-PhotoSen.c
// gcc -Wall -o "%e" "%f" -llgpio -lpthread -g -O0 libMyPi2.a
// Geanyのオプションの"%e"は実行ファイル名, "%f"はソースファイル名のこと

#include <stdio.h>
#include <stdlib.h>         //EXIT_SUCCESS
#include <lgpio.h>          //lgGpiochipOpen,lgGpioRead,etc
#include  "MyPi2.h"         //マイライブラリ
#define PI5         4       // /dev/gpiochip4
#define PI4B        0       // /dev/gpiochip0
#define SEN_RIGHT   4       //右側 反射型フォトセンサ
#define SEN_CENTER  5       //中央 反射型フォトセンサ
#define SEN_LEFT    6       //左側 反射型フォトセンサ
#define SNUM        3       //センサの信号数
#define I2C_BUS     1       // /dev/i2c-1
#define LCD_ADR     0x3e    //LCD スレーブアドレス
#define LCD_IR      0x00    //インストラクションレジスタ

/* グローバル宣言 */
const int inGpio[SNUM] = {SEN_RIGHT,SEN_CENTER,SEN_LEFT};

/* プロトタイプ宣言 */
int ReadSens(int hnd);             //反射型フォトセンサの位置情報の取得
void Sens2Lcd(int hndLcd, int sensors);   //位置情報をLCDに表示

int main(void){
    int hnd,hndLcd;
    int lFlgIn=LG_SET_PULL_NONE;    //内部抵抗を使用しない
    int senNow;                //現在の位置情報
    int senPre=0b0111;         //前回の位置情報
    hnd = lgGpiochipOpen(PI5);
    //hnd = lgGpiochipOpen(PI4B);
    lgGroupClaimInput(hnd, lFlgIn, SNUM, inGpio);  //フォトセンサを入力に設定
    hndLcd = lgI2cOpen(I2C_BUS,LCD_ADR,0);  //LCDのハンドルを取得
    LcdSetup(hndLcd);                       //LCDの初期化

    while(1){
        senNow = ReadSens(hnd);             //反射型フォトセンサの読み込み
        if (senNow!=senPre){                //センサの値が更新されたらLCDに表示する
            LcdClear(hndLcd);               //画面クリア
            Sens2Lcd(hndLcd, senNow);       //LCDにセンサの値を表示
            senPre=senNow;                  //センサの値を更新
        }
        lguSleep(0.001);    //CPU使用率の抑制のため 1ms
    }
    lgI2cClose(hndLcd);
    lgGpiochipClose(hnd);
    return EXIT_SUCCESS;
}

/*関数名   int ReadSens(int hnd)
  引  数   hnd ハンドル
  戻り値   センサの位置情報
  概  要   反射型フォトセンサの位置情報取得
 2bit  1bit    0bit
  左(L)、中央(C)、右(R)  */
int ReadSens(int hnd){
    int i;
    int sensors=0;
    i = lgGpioRead(hnd, SEN_RIGHT);
    if(i == LG_HIGH) {sensors = 1;}     //0bit目 右(R)センサ
    i = lgGpioRead(hnd, SEN_CENTER);
    if(i == LG_HIGH) {sensors += 2;}    //1bit目 中央(C)センサ
    i = lgGpioRead(hnd, SEN_LEFT);
    if(i == LG_HIGH) {sensors += 4;}    //2bit目 左(L)センサ
    return sensors;
}

/*関数名   void Sens2Lcd(int hndLcd, int sensors)
  引  数   hndLcd  LCDのハンドル
  引  数   sensors センサの位置情報
  概  要   反射型フォトセンサの位置情報をLCDに表示
  LCDの表示アドレス
  0123456789abcdef
  L      C       R
  アドレスカウンタの7bit目(ac7)は常に1 */
void Sens2Lcd(int hndLcd, int sensors){
    int ac7=0b10000000;

    lgI2cWriteByteData(hndLcd,LCD_IR,ac7+0xf);  //右(L)センサ
    if(sensors & 0b0001){
        LcdWriteChar(hndLcd, '1');
    }
    else{
        LcdWriteChar(hndLcd, '0');
    }

    lgI2cWriteByteData(hndLcd,LCD_IR,ac7+7);    //中央センサ
    if(sensors & 0b0010){
        LcdWriteChar(hndLcd, '1');
    }
    else{
        LcdWriteChar(hndLcd, '0');
    }

    lgI2cWriteByteData(hndLcd,LCD_IR,ac7+0);    //左センサ
    if(sensors & 0b0100){
        LcdWriteChar(hndLcd, '1');
    }
    else{
        LcdWriteChar(hndLcd, '0');
    }
}
