// 11-Trace.c
// gcc -Wall -o "%e" "%f" -llgpio -lpthread -g -O0 libMyPi2.a
// Geanyのオプションの"%e"は実行ファイル名, "%f"はソースファイル名のこと

#include <stdio.h>
#include <stdlib.h>         //EXIT_SUCCESS
#include <lgpio.h>          //lgGpiochipOpen,lgI2cOpen,etc
#include  "MyPi2.h"         //マイライブラリ
#define PI5         4       // /dev/gpiochip4
#define PI4B        0       // /dev/gpiochip0
#define SW_RED      17      //赤色SW
#define SW_WHITE    27      //白色SW
#define SEN_RIGHT   4       //右側 反射型フォトセンサ
#define SEN_CENTER  5       //中央 反射型フォトセンサ
#define SEN_LEFT    6       //左側 反射型フォトセンサ
#define INNUM       5       //入力信号数
#define MOT_RIGHT2  22      //右モータ BIN2
#define MOT_RIGHT1  23      //右モータ BIN1
#define MOT_LEFT2   24      //左モータ AIN2
#define MOT_LEFT1   25      //左モータ AIN1
#define MNUM        4       //モータの信号数
#define I2C_BUS     1       // /dev/i2c-1
#define LCD_ADR     0x3e    //LCDスレーブアドレス
#define LCD_IR      0x00    //インストラクションレジスタ

/* グローバル宣言 */
//入力用 SW、反射型フォトセンサ
const int inGpio[INNUM] = {SW_RED,SW_WHITE,SEN_RIGHT,SEN_CENTER,SEN_LEFT};
//出力用 モータ駆動IC信号線
const int outGpio[MNUM] = {MOT_RIGHT1,MOT_RIGHT2,MOT_LEFT1,MOT_LEFT2};

/* プロトタイプ宣言 */
int ReadSens(int hnd);          //フォトセンサの位置情報の取得
void Sens2Lcd(int hndLcd, int sensors);   //位置情報をLCDに表示
void RightForward(int hnd);     //右モータ正転
void RightReverse(int hnd);     //右モータ逆転
void LeftForward(int hnd);      //左モータ正転
void LeftReverse(int hnd);      //左モータ逆転
void Forward(int hnd);          //左右のモータの正転
void Reverse(int hnd);          //左右のモータの逆転
void Stop(int hnd);             //左右のモータのストップ
void LcdClrMsg(int hndLcd, char *s); //LCDをクリアして表示

int main(void){
    int hnd,hndLcd;
    int lFlgOut=0;
    int lFlgIn=LG_SET_PULL_NONE;    //内部抵抗を使用しない
    int levels[MNUM]={0,0,0,0};     //出力にグループ化したGPIOの初期値
    int senNow;             //現在の位置情報
    int senPre=0b0111;      //前回の位置情報
    hnd = lgGpiochipOpen(PI5);
    //hnd = lgGpiochipOpen(PI4B);
    lgGroupClaimInput(hnd, lFlgIn, INNUM, inGpio);  //赤色と白色スイッチ、反射型センサを入力に設定
    lgGroupClaimOutput(hnd,lFlgOut,MNUM,outGpio,levels);    //モータ信号を出力に設定
    hndLcd = lgI2cOpen(I2C_BUS,LCD_ADR,0);  //LCDのハンドルを取得
    LcdSetup(hndLcd);                       //LCDの初期化

    LcdClrMsg(hndLcd,"Push White SW.");     //LCDに表示
    while(1){
        if(lgGpioRead(hnd, SW_WHITE) == LG_HIGH){
            while(1){
                senNow = ReadSens(hnd);
                if (senNow != senPre){
                    LcdClear(hndLcd);
                    Sens2Lcd(hndLcd, senNow);
                    senPre = senNow;            //位置情報の更新
                    switch (senNow){
                        case 0b000:
                        case 0b101:
                            Forward(hnd);
                            break;
                        case 0b100:
                        case 0b110:
                            RightForward(hnd);
                            break;
                        case 0b001:
                        case 0b011:
                            LeftForward(hnd);
                            break;
                        default:
                            Stop(hnd);
                            break;
                    }
                }
                lguSleep(0.001);    //CPU使用率の抑制のため 1ms
            }
        }
        lguSleep(0.001);            //CPU使用率の抑制のため 1ms
    }
    lgI2cClose(hndLcd);
    lgGpiochipClose(hnd);
    return EXIT_SUCCESS;
}

/*関数名   int ReadSens(int hnd)
  引  数   hnd ハンドル
  戻り値   センサの位置情報
  概  要   反射型フォトセンサの位置情報取得
 2bit  1bit    0bit
  左(L)、中央(C)、右(R)  */
int ReadSens(int hnd){
    int i;
    int sensors=0;
    i = lgGpioRead(hnd, SEN_RIGHT);
    if(i == LG_HIGH) {sensors = 1;}     //0bit目 右(R)センサ
    i = lgGpioRead(hnd, SEN_CENTER);
    if(i == LG_HIGH) {sensors += 2;}    //1bit目 中央(C)センサ
    i = lgGpioRead(hnd, SEN_LEFT);
    if(i == LG_HIGH) {sensors += 4;}    //2bit目 左(L)センサ
    return sensors;
}

/*関数名   void Sens2Lcd(int hndLcd, int sensors)
  引  数   hndLcd  LCDのハンドル
  引  数   sensors センサの位置情報
  概  要   反射型フォトセンサの位置情報をLCDに表示
  LCDの表示アドレス
  0123456789abcdef
  L      C       R
  アドレスカウンタの7bit目(ac7)は常に1 */
void Sens2Lcd(int hndLcd, int sensors){
    int ac7=0b10000000;

    lgI2cWriteByteData(hndLcd,LCD_IR,ac7+0xf);  //右(L)センサ
    if(sensors & 0b0001){
        LcdWriteChar(hndLcd, '1');
    }
    else{
        LcdWriteChar(hndLcd, '0');
    }

    lgI2cWriteByteData(hndLcd,LCD_IR,ac7+7);    //中央センサ
    if(sensors & 0b0010){
        LcdWriteChar(hndLcd, '1');
    }
    else{
        LcdWriteChar(hndLcd, '0');
    }

    lgI2cWriteByteData(hndLcd,LCD_IR,ac7+0);    //左センサ
    if(sensors & 0b0100){
        LcdWriteChar(hndLcd, '1');
    }
    else{
        LcdWriteChar(hndLcd, '0');
    }
}

/* モータ関係の関数
  右モータ正転  void RightForward(int hnd)
  右モータ逆転  RightReverse(int hnd)
  左モータ正転  LeftForward(int hnd)
  左モータ逆転  void LeftReverse(int hnd)
  左右のモータの正転  void Forward(int hnd)
  左右のモータの逆転  void Reverse(int hnd)
  左右のモータのストップ  void Stop(int hnd)
  引  数   hnd  ハンドル
  戻り値   なし */
void RightForward(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void RightReverse(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void LeftForward(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_HIGH);
}
void LeftReverse(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void Forward(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_HIGH);
}
void Reverse(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void Stop(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT1,LG_HIGH);
}

/*関数名   void LcdClrMsg(int hndLcd, char *s)
  引  数   hndLcd LCDのハンドル
  引  数   s  文字列へのポインタ
  戻り値   なし
  概  要   LCDをクリアして文字列を表示 */
void LcdClrMsg(int hndLcd, char *s){
    LcdClear(hndLcd);
    LcdWriteString(hndLcd, s);
}
