// 12-Distance.c
// gcc -Wall -o "%e" "%f" -llgpio -lpthread -g -O0 libMyPi2.a
// Geanyのオプションの"%e"は実行ファイル名, "%f"はソースファイル名のこと

#include <stdio.h>              //printf,sprintf
#include <stdlib.h>             //EXIT_SUCCESS
#include <lgpio.h>              //lgI2cOpen,lgI2cReadByteData,etc
#include  "MyPi2.h"             //マイライブラリ
#define I2C_BUS         1       // /dev/i2c-1
#define LCD_ADR         0x3e    //LCD スレーブアドレス
#define LCD_IR          0x00    //インストラクションレジスタ
#define GP2Y0E03_ADR    0x40    //GP2Y0E03 スレーブアドレス
#define GP2Y0E03_DIS_U  0x5e    //データレジスタ（上位8bit）
#define GP2Y0E03_DIS_L  0x5f    //データレジスタ（下位4bit）
#define DIS_MIN         4       //測定値の最小値 4cm
#define DIS_MAX         50      //測定値の最大値 50cm

/* プロトタイプ宣言 */
int ReadDistance(int hndGp2);   //距離の取得

int main(void){
    int hndLcd,hndGp2;
    int disCm;
    char s1[17];                //LCD16文字+ヌル文字
    hndLcd = lgI2cOpen(I2C_BUS,LCD_ADR,0);  //LCDのハンドルを取得
    LcdSetup(hndLcd);                       //LCDの初期化
    hndGp2 = lgI2cOpen(I2C_BUS,GP2Y0E03_ADR,0);     //GP2Y0E03のハンドルを取得

    while(1){
        disCm = ReadDistance(hndGp2);
        LcdClear(hndLcd);
        if(DIS_MIN <= disCm && disCm <= DIS_MAX){
            sprintf(s1,"%d cm",disCm);              //測定値を文字列に変換
            LcdWriteString(hndLcd, s1);             //LCDに距離データを表示
            //printf("Distance =%d cm\n",disCm);   //ターミナルにデバック表示
        }else{
            LcdWriteString(hndLcd,"out of range");
            //printf("out of range\n");
        }
        lguSleep(0.5);                  //0.5秒待ち時間
    }
    lgI2cClose(hndGp2);
    lgI2cClose(hndLcd);
    return EXIT_SUCCESS;
}

/*関数名   int ReadDistance(int hndGp2)
  引  数   hndGp2 距離センサGP2Y0E03のハンドル
  戻り値   距離データ(整数) cm
  概  要   距離センサから距離データを取得 */
int ReadDistance(int hndGp2){
    int disU8bit,disL4bit,dis12bit;
    disU8bit = lgI2cReadByteData(hndGp2, GP2Y0E03_DIS_U);    //上位8bit取得
    disL4bit = lgI2cReadByteData(hndGp2, GP2Y0E03_DIS_L);    //下位4bit取得
    dis12bit = (disU8bit<<4)|disL4bit;  //12bit=上位8bit+下位4bit
    dis12bit = dis12bit>>6;             //6bit右シフト
    return dis12bit;
}
