// 12-Safety.c
// gcc -Wall -o "%e" "%f" -llgpio -lpthread -g -O0 libMyPi2.a
// Geanyのオプションの"%e"は実行ファイル名, "%f"はソースファイル名のこと

#include <stdio.h>
#include <stdlib.h>             //EXIT_SUCCESS
#include <lgpio.h>              //lgI2cOpen,lgTxServo,etc
#include  "MyPi2.h"             //マイライブラリ
#define PI5             4       // /dev/gpiochip4
#define PI4B            0       // /dev/gpiochip0
#define LED             20      //LED
#define SERVO           13      //GPIO13をSERVOと定義
#define SW_RED          17      //赤色SW
#define SW_WHITE        27      //白色SW
#define SEN_RIGHT       4       //右側 反射型フォトセンサ
#define SEN_CENTER      5       //中央 反射型フォトセンサ
#define SEN_LEFT        6       //左側 反射型フォトセンサ
#define INNUM           5       //入力信号数
#define MOT_RIGHT2      22      //右モータ BIN2
#define MOT_RIGHT1      23      //右モータ BIN1
#define MOT_LEFT2       24      //左モータ AIN2
#define MOT_LEFT1       25      //左モータ AIN1
#define MNUM            4       //モータの信号数
#define I2C_BUS         1       // /dev/i2c-1
#define LCD_ADR         0x3e    //LCD スレーブアドレス
#define LCD_IR          0x00    //インストラクションレジスタ
#define GP2Y0E03_ADR    0x40    //GP2Y0E03 スレーブアドレス
#define GP2Y0E03_DIS_U  0x5e    //データレジスタ（上位8bit）
#define GP2Y0E03_DIS_L  0x5f    //データレジスタ（下位4bit）
#define GP2Y0E03_MDN    0x3f    //中央値(median)フィルタレジスタ
#define GP2Y0E03_MDN_5  0x10    //中央値演算のデータ数5
#define GP2Y0E03_RSP    120     //応答時間(ms)
#define DANGER          15      //障害物の検出距離(cm)
#define DIS_DIFF        3       //距離差(cm)
#define SM_RGHT         1000    //サーボモータ右向きパルス幅(us)
#define SM_FRNT         1400    //サーボモータ正面向きパルス幅(us)
#define SM_LFT          1800    //サーボモータ左向きパルス幅(us)
#define SM_INTVL        400     //サーボモータの制御の時間間隔(ms)

/* グローバル宣言 */
//入力用 SW、反射型フォトセンサ
const int inGpio[5] = {SW_RED,SW_WHITE,SEN_RIGHT,SEN_CENTER,SEN_LEFT};
//出力用 モータ駆動IC信号線
const int outGpio[4] = {MOT_RIGHT1,MOT_RIGHT2,MOT_LEFT1,MOT_LEFT2};
/* プロトタイプ宣言 */
int ReadSens(int hnd);             //フォトセンサの位置情報の取得
void Sens2Lcd(int hndLcd, int sensors);   //位置情報をLCDに表示
void RightForward(int hnd);    //右モータ正転
void RightReverse(int hnd);    //右モータ逆転
void LeftForward(int hnd);     //左モータ正転
void LeftReverse(int hnd);     //左モータ逆転
void Forward(int hnd);         //左右のモータの正転
void Reverse(int hnd);         //左右のモータの逆転
void Stop(int hnd);            //左右のモータのストップ
void LcdClrMsg(int hndLcd, char *s);  //LCDをクリアして表示
int ReadDistance(int hndGp2);   //距離の取得
unsigned int millis(void);      //UNIX時間msの取得

int main(void){
    int hnd;                    //GPIO用
    int hndLcd;                 //LCD用
    int hndGp2;                 //距離センサGP2Y0E03用
    int i;
    int lFlgOut=0;
    int lFlgIn=LG_SET_PULL_NONE;    //内部抵抗を使用しない
    int levels[MNUM]={0,0,0,0};     //出力にグループ化したGPIOの初期値
    char s1[17];                //LCD16文字+ヌル文字
    int positionNow = 0;        //現在の位置情報
    int distanceNow = 0;        //現在の距離(cm)
    int distancePre = 100;      //前回の距離(cm)
    unsigned int timeNow1;      //距離センサ用タイムスタンプ
    unsigned int timePre1 =0;
    unsigned int timeNow2;      //サーボモータ用タイムスタンプ
    unsigned int timePre2 =0;
    int pulseWidth;             //サーボ信号のパルス幅 (us)
    int servoFrequency=50;      //周波数 50Hz
    int servoOffset=0;          //オフセットなし
    int servoCycles=5;          //5パルス出力

    hnd = lgGpiochipOpen(PI5);
    //hnd = lgGpiochipOpen(PI4B);
    lgGpioClaimOutput(hnd,lFlgOut,LED,LG_LOW);      //LED
    lgGroupClaimInput(hnd, lFlgIn, INNUM, inGpio);  //赤色と白色スイッチ、反射型センサを入力に設定
    lgGroupClaimOutput(hnd,lFlgOut,MNUM,outGpio,levels);    //モータ信号を出力に設定

    hndLcd = lgI2cOpen(I2C_BUS,LCD_ADR,0);  //LCDのハンドルを取得
    LcdSetup(hndLcd);                       //LCDの初期化
    hndGp2 = lgI2cOpen(I2C_BUS,GP2Y0E03_ADR,0); //GP2Y0E03のハンドルを取得
    lgI2cWriteByteData(hndGp2,GP2Y0E03_MDN,GP2Y0E03_MDN_5); //中央値演算のデータ数5

    //サーボモータの動作テスト
    lgGpioClaimOutput(hnd,lFlgOut,SERVO,LG_LOW); //サーボ信号を出力に設定
    pulseWidth=SM_RGHT;                         //サーボモータ右側に向ける
    lgTxServo(hnd,SERVO,pulseWidth,servoFrequency,servoOffset,servoCycles);
    lguSleep(1);                                //１秒待つ
    pulseWidth=SM_LFT;                          //サーボモータ左側に向ける
    lgTxServo(hnd,SERVO,pulseWidth,servoFrequency,servoOffset,servoCycles);
    lguSleep(1);
    pulseWidth=SM_FRNT;                         //サーボモータ正面に向ける
    lgTxServo(hnd,SERVO,pulseWidth,servoFrequency,servoOffset,servoCycles);
    lguSleep(1);

    LcdClrMsg(hndLcd,"Push White SW.");          //LCDに表示
    do{
        i = lgGpioRead(hnd, SW_WHITE);
        lguSleep(0.001);                        //CPU使用率の抑制のため
    }while(i != LG_HIGH);

    while(1){
        timeNow1 = millis();
        if(timeNow1 > timePre1 + GP2Y0E03_RSP){ //距離の測定
            timePre1 = timeNow1;
            distancePre = distanceNow;          //距離の更新
            distanceNow = ReadDistance(hndGp2);
        }
        if(distanceNow != distancePre){         //LCDに距離の表示
            LcdClear(hndLcd);
            sprintf(s1,"%d cm",distanceNow);
            LcdWriteString(hndLcd, s1);
        }
        if(distanceNow <= DANGER){      //障害物を検出したとき
            Stop(hnd);                  //自走ロボットストップ
            pulseWidth=0;
            lgTxServo(hnd,SERVO,pulseWidth,servoFrequency,servoOffset,servoCycles);  //サーボモータストップ
            i = abs(distanceNow - distancePre);
            if(DIS_DIFF <= i){
                lguSleep(0.5);
                system("rpicam-still -t 300 -o /home/my-pi/MyApp2/$(date +'%Y%m%d-%H%M%S').jpg");
            }
        }else{      //障害物を検出していないとき
            //RCサーボモータの制御
            positionNow = ReadSens(hnd);
            timeNow2 = millis();
            if(timeNow2 > timePre2 + SM_INTVL){ //設定した時間間隔を経過した場合
                switch (positionNow){
                case 0b000:                 //ノーマルポジション
                case 0b101:                 //白線上にいる
                    pulseWidth = SM_FRNT;   //サーボモータ正面に向ける
                    break;
                case 0b100:                 //白線の右側にいる
                case 0b110:
                    pulseWidth = SM_LFT;    //サーボモータ左側に向ける
                    break;
                case 0b001:                 //白線の左側にいる
                case 0b011:
                    pulseWidth = SM_RGHT;   //サーボモータ右側に向ける
                    break;
                default:
                    pulseWidth=0;           //サーボモータストップ
                    break;
                }
                lgTxServo(hnd,SERVO,pulseWidth,servoFrequency,servoOffset,servoCycles); //サーボ信号出力
                timePre2 = timeNow2;
            }

            //タイヤの制御
            switch (positionNow){
            case 0b000:         //ノーマルポジション
            case 0b101:         //白線上にいる
                Forward(hnd);
                break;
            case 0b100:         //白線の右側にいる
            case 0b110:
                RightForward(hnd);
                break;
            case 0b001:         //白線の左側にいる
            case 0b011:
                LeftForward(hnd);
                break;
            default:
                Stop(hnd);
                pulseWidth=0;   //サーボモータストップ
                lgTxServo(hnd,SERVO,pulseWidth,servoFrequency,servoOffset,servoCycles);
                break;
            }
        }
        lguSleep(0.001);        //CPU使用率の抑制のため 1ms
    }
    lgI2cClose(hndGp2);
    lgI2cClose(hndLcd);
    lgGpiochipClose(hnd);
    return EXIT_SUCCESS;
}

/*関数名   millis(void)
  引  数   なし
  戻り値   ミリ秒単位のUNIX時間
  概  要   ミリ秒単位のUNIX時間を出力する */
unsigned int millis(void){
    unsigned int epochMill;   //milliseconcd
    unsigned long epochNano;  //nanosecond
    epochNano = lguTimestamp();
    epochMill = (unsigned int)(epochNano / 1000000); //nsからmsに変換
    return epochMill;
}

/*関数名   int ReadDistance(int hndGp2)
  引  数   hndGp2 距離センサGP2Y0E03のハンドル
  戻り値   距離(整数) cm
  概  要   距離センサから距離データを取得 */
int ReadDistance(int hndGp2){
    int disU8bit,disL4bit,dis12bit;
    disU8bit = lgI2cReadByteData(hndGp2, GP2Y0E03_DIS_U);    //上位8bit取得
    disL4bit = lgI2cReadByteData(hndGp2, GP2Y0E03_DIS_L);    //下位4bit取得
    dis12bit = (disU8bit<<4)|disL4bit;  //12bit=上位8bit+下位4bit
    dis12bit = dis12bit>>6;             //6bit右シフト
    return dis12bit;
}

/*関数名   int ReadSens(int hnd)
  引  数   hnd ハンドル
  戻り値   センサの位置情報
  概  要   反射型フォトセンサの位置情報取得
 2bit  1bit    0bit
  左(L)、中央(C)、右(R)  */
int ReadSens(int hnd){
    int i;
    int sensors=0;
    i = lgGpioRead(hnd, SEN_RIGHT);
    if(i == LG_HIGH) {sensors = 1;}     //0bit目 右(R)センサ
    i = lgGpioRead(hnd, SEN_CENTER);
    if(i == LG_HIGH) {sensors += 2;}    //1bit目 中央(C)センサ
    i = lgGpioRead(hnd, SEN_LEFT);
    if(i == LG_HIGH) {sensors += 4;}    //2bit目 左(L)センサ
    return sensors;
}

/*関数名   void Sens2Lcd(int hndLcd, int sensors)
  引  数   hndLcd  LCDのハンドル
  引  数   sensors センサの位置情報
  概  要   反射型フォトセンサの位置情報をLCDに表示
  LCDの表示アドレス
  0123456789abcdef
  L      C       R
  アドレスカウンタの7bit目(ac7)は常に1 */
void Sens2Lcd(int hndLcd, int sensors){
    int ac7=0b10000000;

    lgI2cWriteByteData(hndLcd,LCD_IR,ac7+0xf);  //右(L)センサ
    if(sensors & 0b0001){
        LcdWriteChar(hndLcd, '1');
    }
    else{
        LcdWriteChar(hndLcd, '0');
    }

    lgI2cWriteByteData(hndLcd,LCD_IR,ac7+7);    //中央センサ
    if(sensors & 0b0010){
        LcdWriteChar(hndLcd, '1');
    }
    else{
        LcdWriteChar(hndLcd, '0');
    }

    lgI2cWriteByteData(hndLcd,LCD_IR,ac7+0);    //左センサ
    if(sensors & 0b0100){
        LcdWriteChar(hndLcd, '1');
    }
    else{
        LcdWriteChar(hndLcd, '0');
    }
}

/* モータ関係の関数
  右モータ正転  void RightForward(int hnd)
  右モータ逆転  RightReverse(int hnd)
  左モータ正転  LeftForward(int hnd)
  左モータ逆転  void LeftReverse(int hnd)
  左右のモータの正転  void Forward(int hnd)
  左右のモータの逆転  void Reverse(int hnd)
  左右のモータのストップ  void Stop(int hnd)
  引  数   hnd  ハンドル
  戻り値   なし */
void RightForward(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void RightReverse(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void LeftForward(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_HIGH);
}
void LeftReverse(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void Forward(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_LOW);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT2,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT1,LG_HIGH);
}
void Reverse(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_LOW);
    lgGpioWrite(hnd,MOT_LEFT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT1,LG_LOW);
}
void Stop(int hnd){
    lgGpioWrite(hnd,MOT_RIGHT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_RIGHT1,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT2,LG_HIGH);
    lgGpioWrite(hnd,MOT_LEFT1,LG_HIGH);
}

/*関数名   void LcdClrMsg(int hndLcd, char *s)
  引  数   hndLcd LCDのハンドル
  引  数   s  文字列へのポインタ
  戻り値   なし
  概  要   LCDをクリアして文字列を表示 */
void LcdClrMsg(int hndLcd, char *s){
    LcdClear(hndLcd);
    LcdWriteString(hndLcd, s);
}
