// 12-Servo.c
// gcc -Wall -o "%e" "%f" -llgpio -lpthread -g -O0 libMyPi2.a
// Geanyのオプションの"%e"は実行ファイル名, "%f"はソースファイル名のこと

#include <stdio.h>          //printf,sprintf
#include <stdlib.h>         //EXIT_SUCCESS
#include <lgpio.h>          //lgGpiochipOpen,lgTxServo,etc
#include  "MyPi2.h"         //マイライブラリ
#define PI5         4       // /dev/gpiochip4
#define PI4B        0       // /dev/gpiochip0
#define SERVO       13      //GPIO13をSERVOと定義
#define I2C_BUS     1       // /dev/i2c-1
#define LCD_ADR     0x3e    //LCD スレーブアドレス
#define LCD_IR      0x00    //インストラクションレジスタ

/* プロトタイプ宣言 */
void LcdClrMsg(int hndLcd, char *s);    //ディスプレイクリア＋文字列表示

int main(void){
    int hnd,hndLcd;
    int lFlgOut=0;
    char s1[17];                //LCD16文字+ヌル文字
    int kbNum;
    float tON;
    int pulseWidth;             //制御信号のパルス幅 (us)
    int servoFrequency=50;      //周波数 50Hz
    int servoOffset=0;          //オフセット時間0(us)
    int servoCycles=0;          //連続出力

    hnd = lgGpiochipOpen(PI5);
    //hnd = lgGpiochipOpen(PI4B);
    lgGpioClaimOutput(hnd,lFlgOut,SERVO,LG_LOW);
    hndLcd = lgI2cOpen(I2C_BUS,LCD_ADR,0);  //LCDのハンドルを取得
    LcdSetup(hndLcd);                       //LCDの初期化
    LcdClrMsg(hndLcd,"Servo Motor Test");
    while(1){
        printf("パルスオン時間(0.1ms*) として0から30までの整数を入力 >>> ");
        scanf("%d",&kbNum);
        if (0<=kbNum && kbNum<=30){
            pulseWidth=kbNum*100;
            lgTxServo(hnd,SERVO,pulseWidth,servoFrequency,\
            servoOffset,servoCycles); //信号出力
            tON = kbNum*0.1;
            printf("tON = %2.1f ms\n\n",tON);
            sprintf(s1,"tON = %2.1f ms",tON);
            LcdClrMsg(hndLcd,s1);
            lguSleep(0.1);
            pulseWidth=0;
            lgTxServo(hnd,SERVO,pulseWidth,servoFrequency,\
            servoOffset,servoCycles); //信号停止
        }else{
            pulseWidth=0;
            lgTxServo(hnd,SERVO,pulseWidth,servoFrequency,\
            servoOffset,servoCycles); //信号停止
            printf("値が範囲外です\n\n");
            LcdClrMsg(hndLcd,"error");
        }
    }
    lgI2cClose(hndLcd);
    lgGpiochipClose(hnd);
    return EXIT_SUCCESS;
}

/*関数名   void LcdClrMsg(int hndLcd, char *s)
  引  数   hndLcd  LCDのルンドル
  引  数   s 文字列へのポインタ
  戻り値   なし
  概  要   LCDをクリアして文字列を表示 */
void LcdClrMsg(int hndLcd, char *s){
    LcdClear(hndLcd);
    LcdWriteString(hndLcd, s);
}
