// 6-Thread01.c
// gcc -Wall -o "%e" "%f" -llgpio -lpthread -g -O0
// Geanyのオプションの"%e"は実行ファイル名, "%f"はソースファイル名のこと

#include <stdio.h>      //printf
#include <stdlib.h>     //EXIT_SUCCESS
#include <lgpio.h>      //lgThreadStart,lgThreadStop,etc
#define PI5     4       // /dev/gpiochip4
#define PI4B    0       // /dev/gpiochip0
#define LED0    23
#define LED1    22
#define LED2    25
#define LNUM    3       //LEDの数
/* プロトタイプ宣言 */
void *Led2Hz(void *userdata); //2Hz信号
void *Led3Hz(void *userdata); //3Hz信号
/* グローバル変数　*/
int g_hnd;              //handle

int main (void){
    int i,t;
    int userdata=10000;    //点滅時間（期間）ms
    pthread_t *p2Hz,*p3Hz;
    int lFlgOut=0;
    int leds[LNUM]={LED0,LED1,LED2};
    int levels[LNUM]={0,0,0};
    uint64_t gMsk=0b111;
    g_hnd = lgGpiochipOpen(PI5);
    //g_hnd = lgGpiochipOpen(PI4B);
    lgGroupClaimOutput(g_hnd,lFlgOut,LNUM,leds,levels);

    p2Hz = lgThreadStart(Led2Hz,&userdata);
    p3Hz = lgThreadStart(Led3Hz,&userdata);
    t = userdata;
    for(i = 0; i < t/1000; i++){
        //printf("LED0: #%d\n",i);          //デバッグ用
        lgGpioWrite(g_hnd,LED0,LG_HIGH);    //点灯
        lguSleep(0.5);                      //0.5秒時間待ち
        lgGpioWrite(g_hnd,LED0,LG_LOW);     //消灯
        lguSleep(0.5);                      //0.5秒時間待ち
    }
    printf("End of main\n");
    lguSleep(0.1);
    lgThreadStop(p2Hz);//各スレッドの終了
    lgThreadStop(p3Hz);
    lgGroupWrite(g_hnd,LED0,0,gMsk);  //LEDを消灯
    lgGpiochipClose(g_hnd);
    return EXIT_SUCCESS;
}

//関数名  void *Led2Hz(void *userdata)
//引数    userdata  時間(ms)
//戻り値  なし
//概要    LED1を2Hzで引数の時間(ms)点滅させる
void *Led2Hz(void *userdata){
    int i,t;
    t = *(int*) userdata;
    for(i = 0; i < t/500; i++){
        //printf("LED1: #%d\n",i);          //デバッグ用
        lgGpioWrite(g_hnd,LED1,LG_HIGH);    //点灯
        lguSleep(0.25);                     //0.25秒時間待ち
        lgGpioWrite(g_hnd,LED1,LG_LOW);     //消灯
        lguSleep(0.25);                     //0.25秒時間待ち
    }
    printf("End of Led2Hz\n");
    pthread_exit(NULL);
}

//関数名  void *Led3Hz(void *userdata)
//引数    userdata  時間(ms)
//戻り値  なし
//概要    LED2を3Hzで引数の時間(ms)点滅させる
void *Led3Hz(void *userdata){
    int i,t;
    t = *(int*) userdata;
    for(i = 0; i < (int) t/333; i++){
        //printf("LED2: #%d\n",i);          //デバッグ用
        lgGpioWrite(g_hnd,LED2,LG_HIGH);    //点灯
        lguSleep(0.167);                    //0.167秒時間待ち
        lgGpioWrite(g_hnd,LED2,LG_LOW);     //消灯
        lguSleep(0.167);                    //0.167秒時間待ち
    }
    printf("End of Led3Hz\n");
    pthread_exit(NULL);
}
