// 8-Dac01.c
// gcc -Wall -o "%e" "%f" -llgpio -lpthread -g -O0 libMyPi2.a
// Geanyのオプションの"%e"は実行ファイル名, "%f"はソースファイル名のこと

#include <stdio.h>      //printf,scanf
#include <stdlib.h>     //EXIT_SUCCESS
#include <lgpio.h>      //lgSpiOpen,lgSpiWrite,etc

//SPIバスインタフェース
#define SPI_BUS     0       // /dev/spidev0
#define SPI_SS1     1       //DACのスレーブセレクト番号
#define SPI_SPEED   500000  //クロック信号の周波数
#define SPI_MODE    0       //SPIモード0

//D/Aコンバータ  MCP4922
#define MCP4922_DACA    0           //DACAチャンネル
#define MCP4922_DACB    0b10000000  //DACBチャンネル
#define MCP4922_GAx1    0b00100000  //ゲイン1倍
#define MCP4922_SHDN    0b00010000  //VOUTの出力を有効

int Mcp4922Write(int hndDac, unsigned char dacCh, unsigned short dacCode);

int main (void){
    int hndDac;
    unsigned int dacCode;
    hndDac = lgSpiOpen(SPI_BUS, SPI_SS1, SPI_SPEED, SPI_MODE); //DACのハンドルを取得
    while(1){
        printf("16進数で0からfffまでの値を入力してください >>>");
        scanf("%x",&dacCode);
        if(0<=dacCode && dacCode <=0xfff){
            printf("VoutA,VoutB = %5.3f V\n\n", ((3.3/4096) * dacCode));
            Mcp4922Write(hndDac,MCP4922_DACA,dacCode);
            Mcp4922Write(hndDac,MCP4922_DACB,dacCode);
        }else{
            printf("値が範囲外です\n\n");
            Mcp4922Write(hndDac,MCP4922_DACA,0);
            Mcp4922Write(hndDac,MCP4922_DACB,0);
        }
    }
    lgSpiClose(hndDac);
    return EXIT_SUCCESS;
}

//関数名  int Mcp4922Write(int hndDac,unsigned char dacCh, unsigned short dacCode)
//引  数  hndDac lgSpiOpen関数で取得したハンドル
//引  数  dacCh DACチャンネル DACA=0, DACB=0b10000000
//引  数  dacCode 12bit DACコード 0000-0fff
//戻り値  lgSpiWrite関数の戻り値
//概  要  0Vから約3.3Vの範囲で、12bit分解能でアナログ電圧を出力する
int Mcp4922Write(int hndDac,unsigned char dacCh, unsigned short dacCode)
{
    int ret;
    char txBuf[2]={0,0};
    if(dacCh == MCP4922_DACB){    //DACBチャンネルを指定
        txBuf[0] |= MCP4922_DACB;
    }
    txBuf[0] = txBuf[0]|MCP4922_GAx1|MCP4922_SHDN; //config bitをセット
    txBuf[0]|= dacCode>>8;        //12bit DACコードの上位4ビットをセット
    txBuf[1] = dacCode&0x00ff;    //12bit DACコードの下位8ビットをセット
    ret = lgSpiWrite(hndDac, txBuf, sizeof(txBuf)); //SPIバスへ出力
    return ret;
}
